
//
//
// Project       : Vorpal-Mail
//
// Source        : array.cc
//
// Version       : 1.0
//
// Creator       : Blane Bramble                    Date          : 27th October 2000
// Amended       : Blane Bramble                    Date          : 7th November 2000
//
// License       : Copyright Virtua-Web
//
// Description   :
//
// Class to support extendible arrays (initially for strings). This is needed
// by the configuration file/virtual domain support for lists of users etc.
//
//
// Changelog     :
//
// 7 Nov 2000
// Added touch routines to allow a search routine to exclude array items from further
// searches if they have already been matched (to prevent circular references in
// users and lists etc).
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "array.h"

STRING_ARRAY::STRING_ARRAY(void)
{
array = NULL;
size = 0;
}

STRING_ARRAY::~STRING_ARRAY()
{
int n;
for(n = 0; n<size;n++)
	free(array[n]);
if (array != NULL)
	delete array;
}

int STRING_ARRAY::add(char *s)
{
char **old;
int n;
old = array;
array = new char *[size+1];
for(n=0;n<size;n++)
	array[n] = old[n];
if (old != NULL)
	delete old;
array[size] = strdup(s);
return size++;
}

int STRING_ARRAY::find(char *s)
{
int n;
for(n=0;n<size;n++)
	if (strcasecmp(array[n], s) == 0)
		return n;

return -1;
}

char *STRING_ARRAY::get(int n)
{
if (size == 0 || array == NULL || n >= size)
	return NULL;
return array[n];
}

void STRING_ARRAY::remove(int n)
{
char **newarray;

if (size > 0)
	{
	int i;

	if (size == 1)
		{
		free(array[0]);
		delete array;
		array = NULL;
		size = 0;
		}
	else
		{
		newarray = new char *[size-1];

		free(array[n]);			// release the pointer

		for(i=0;i<n;i++)
			newarray[i] = array[i];

		for(i=n+1;i<size;i++)
			newarray[i-1] = array[i];

		delete array;
		array = newarray;
		size--;
		}
	}
}

void STRING_ARRAY::replace(int n, char *s)
{
if (n < size)
	{
	free(array[n]);
	array[n] = strdup(s);
	}
}

