
//
//
// Project       : Vorpal-Mail
//
// Source        : list.cc
//
// Creator       : Blane Bramble                    Date          : 26th October 2000
// Amended       : Blane Bramble                    Date          : 27th October 2000
//
// License       : Copyright Virtua-Web
//
// Implementation of linked list classes for Vorpal Mail. Implements a LIST class which uses a LISTITEM class,
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "list.h"

LISTITEM::LISTITEM(char *k, void *d)
{
data = d;
if (k != NULL)
	key = strdup(k);
else
	key = NULL;
prev = NULL;
next = NULL;
itouch = 0;
}

LISTITEM::~LISTITEM()
{
if (key != NULL)
	free(key);
}


LIST::LIST(void)
{
head = NULL;
tail = NULL;
}

LIST::~LIST()
{
LISTITEM *chain, *next;
chain = head;
while(chain != NULL)
	{
	next = chain->getnext();
	delete chain;
	chain = next;
	}
}

LISTITEM *LIST::add(char *k, void *d)
{
LISTITEM *item;

if (d == NULL || (item = new LISTITEM(k, d)) == NULL)
	return NULL;

if (head == NULL)		// first item in list!
	{
	head = item;
	tail = item;
	}
else
	{
	if (tail != NULL)	// last item exists, so add to list
		{
		tail->setnext(item);
		item->setprev(tail);
		}
	tail = item;		// and make last
	}
return item;
}

LISTITEM *LIST::find(char *k)
{
LISTITEM *item;

item = head;
while(item != NULL)
	{
	if (strcasecmp(k, item->getkey()) == 0)
		return item;
	item = item->getnext();
	}
return NULL;
}

LISTITEM *LIST::finddata(void *d)
{
LISTITEM *item;

item = head;
while(item != NULL)
	{
	if (d == item->getdata())
		return item;
	item = item->getnext();
	}
return NULL;
}

void *LIST::del(LISTITEM *i)
{
void *d;
if (i->getprev() != NULL)		// if there is a link before me
	(i->getprev())->setnext(i->getnext());
if (i->getnext() != NULL)		// if there is a link after me
	(i->getnext())->setprev(i->getprev());
if (head == i)
	head = i->getnext();
if (tail == i)
	head = i->getprev();
d = i->getdata();
delete i;
return d;
}

void LIST::untouch(void)
{
LISTITEM *item;

item = head;
while(item != NULL)
	{
	item->untouch();
	item = item->getnext();
	}
}

LISTITEM *LIST::touchfind(char *s)
{
LISTITEM *item;

item = head;
while(item != NULL)
	{
	if (!item->touched() && strcasecmp(s, item->getkey()) == 0)
		{
		item->touch();
		return item;
		}
	item = item->getnext();
	}
return NULL;
}

